@@ -33,6 +33,10 @@
 #include "clockdomain.h"
 #include "config.h"
 
+#if defined(OS2) && defined(C_SDL2)
+#undef VERSION
+#endif
+
 #if defined(C_HAVE_LINUX_KVM) && (C_TARGETCPU == X86 || C_TARGETCPU == X86_64)
 # define C_HAVE_LINUX_KVM_X86
 #endif
@@ -185,8 +189,9 @@ extern bool				sse2_available;
 extern bool				avx2_available;
 #endif
 
-void					MSG_Add(const char*,const char*); //add messages to the internal languagefile
-const char*				MSG_Get(char const *);     //get messages from the internal languagefile
+void                    MSG_Add(const char*,const char*);      // Add messages to the internal languagefile
+const char*             MSG_Get(char const *);                 // Get messages from the internal languagefile
+std::string             formatString(const char* format, ...); // Generates a formatted string using a format specifier and variable arguments.
 
 void					DOSBOX_RunMachine();
 void					DOSBOX_SetLoop(LoopHandler * handler);
@@ -359,23 +364,10 @@ private:
     SaveState(const SaveState&);
     SaveState& operator=(const SaveState&);
 
-    class RawBytes
-    {
-    public:
-        RawBytes() {}
-        void set(const std::string& stream);
-        std::string get() const; //throw (Error)
-        bool dataAvailable() const;
-    private:
-        bool dataExists = false; //determine whether set method (even with empty string) was called
-        mutable std::string bytes; //
-    };
-
     struct CompData
     {
         CompData(Component& cmp) : comp(cmp) {}
         Component& comp;
-        std::vector<RawBytes> rawBytes = std::vector<RawBytes>(MAX_PAGE * SLOT_COUNT);
     };
 
     typedef std::map<std::string, CompData> CompEntry;
