@@ -45,11 +45,13 @@ extern DOS_Shell * first_shell;
 
 const std::map<int, std::string> langcp_map {
 	{437, "en_US"},
-	{850, "de_DE"},
-	{857, "tr_TR"},
+	//{850, "de_DE"},
+    {852, "hu_HU"},
+    {857, "tr_TR"},
 	{858, "es_ES"},
 	{859, "fr_FR"},
 	{860, "pt_BR"},
+	{866, "ru_RU"},
 	{932, "ja_JP"},
 	{936, "zh_CN"},
 	{949, "ko_KR"},
