@@ -22,13 +22,16 @@
 
 
 #define VOODOO_INITIAL_LFB	0xd0000000
+#define VOODOO_LFB_SIZE		0x01000000 /* VOODOO_PAGES == 4096   (4096 << 12) 16MB size must be a power of 2 */
+#define VOODOO_LFB_ALIGNMASK	( ~(VOODOO_LFB_SIZE - 1u) ) /* 16MB align */
 #define VOODOO_REG_PAGES	1024
 #define VOODOO_LFB_PAGES	1024
 #define VOODOO_TEX_PAGES	2048
-#define VOODOO_PAGES (VOODOO_REG_PAGES+VOODOO_LFB_PAGES+VOODOO_TEX_PAGES)
+#define VOODOO_PAGES		(VOODOO_REG_PAGES+VOODOO_LFB_PAGES+VOODOO_TEX_PAGES)
 
+static_assert( VOODOO_LFB_ALIGNMASK == 0xFF000000ul, "Voodoo LFB error" );
 
-#define VOODOO_EMU_TYPE_OFF			0
+#define VOODOO_EMU_TYPE_OFF		0
 #define VOODOO_EMU_TYPE_SOFTWARE	1
 #define VOODOO_EMU_TYPE_ACCELERATED	2
 
