@@ -511,7 +511,7 @@ void RENDER_Reset( void ) {
 		goto forcenormal;
 
 	if(render.scale.prompt && (!dblh || !dblw) && !((dblh || dblw) && !render.scale.hardware) && scalerOpTV != render.scale.op && scaler != "none" && strncasecmp(scaler.c_str(), "normal", 6) && !render.scale.forced && sdl.desktop.want_type != SCREEN_TTF) {
-		std::string message = "This scaler may not work properly or have undesired effect:\n\n"+scaler+"\n\nDo you want to force load the scaler?";
+		std::string message = formatString(MSG_Get("SCALER_LOAD_WARN"), scaler.c_str());
 		render.scale.forced = systemmessagebox("Loading scaler", message.c_str(), "yesno","question", 1);
 		render.scale.prompt = false;
 	}
@@ -1161,6 +1161,7 @@ void RENDER_OnSectionPropChange(Section *x) {
 
     bool p_doublescan = vga.draw.doublescan_set;
     bool p_char9 = vga.draw.char9_set;
+    bool p_modeswitch = vga.draw.modeswitch_set;
     int p_aspect = render.aspect;
 
     std::string s_aspect = section->Get_string("aspect");
@@ -1175,16 +1176,26 @@ void RENDER_OnSectionPropChange(Section *x) {
 
     vga.draw.doublescan_set=section->Get_bool("doublescan");
     vga.draw.char9_set=section->Get_bool("char9");
+    
+#if C_SDL2
+	vga.draw.modeswitch_set=section->Get_bool("modeswitch");
+#endif
 
-    if (render.aspect != p_aspect || vga.draw.doublescan_set != p_doublescan || vga.draw.char9_set != p_char9)
+    if (render.aspect != p_aspect || vga.draw.doublescan_set != p_doublescan || \
+		vga.draw.char9_set != p_char9 || vga.draw.modeswitch_set != p_modeswitch)
         RENDER_CallBack(GFX_CallBackReset);
-    if (vga.draw.doublescan_set != p_doublescan || vga.draw.char9_set != p_char9)
+    if (vga.draw.doublescan_set != p_doublescan || vga.draw.char9_set != p_char9 || \
+		vga.draw.modeswitch_set != p_modeswitch)
         VGA_StartResize();
 
     mainMenu.get_item("vga_9widetext").check(vga.draw.char9_set).refresh_item(mainMenu);
     mainMenu.get_item("doublescan").check(vga.draw.doublescan_set).refresh_item(mainMenu);
     mainMenu.get_item("mapper_aspratio").check(render.aspect).refresh_item(mainMenu);
 
+#if C_SDL2
+    mainMenu.get_item("modeswitch").check(vga.draw.modeswitch_set).refresh_item(mainMenu);
+#endif
+
 #if C_XBRZ
     xBRZ_Change_Options(section);
 #endif
@@ -1359,6 +1370,11 @@ void RENDER_Init() {
 
     vga.draw.doublescan_set=section->Get_bool("doublescan");
     vga.draw.char9_set=section->Get_bool("char9");
+
+#if C_SDL2
+	vga.draw.modeswitch_set=section->Get_bool("modeswitch");
+#endif
+
 	eurAscii = section->Get_int("euro");
 	if (eurAscii != -1 && (eurAscii < 33 || eurAscii > 255)) {
 		LOG_MSG("Euro ASCII value has to be between 33 and 255\n");
@@ -1444,6 +1460,10 @@ void RENDER_Init() {
     mainMenu.get_item("doublescan").check(vga.draw.doublescan_set).refresh_item(mainMenu);
     mainMenu.get_item("mapper_aspratio").check(render.aspect).refresh_item(mainMenu);
 
+#if C_SDL2
+    mainMenu.get_item("modeswitch").check(vga.draw.modeswitch_set).refresh_item(mainMenu);
+#endif
+
     RENDER_UpdateFrameskipMenu();
 
     /* BUG FIX: Some people's dosbox-x.conf files have frameskip=-1 WTF?? */
