@@ -1289,6 +1289,20 @@ void INT10_GetCursorPos(uint8_t *row, uint8_t*col, const uint8_t page)
 }
 
 void INT10_SetCursorPos(uint8_t row,uint8_t col,uint8_t page) {
+    // Get the dimensions
+    BIOS_NCOLS;
+    BIOS_NROWS;
+
+    // EGA/VGA: Emulate a VGA BIOS that range-checks at least the row.
+    //          The DOS installer for Elder Scrolla Arena is obviously trying to put something on the last row of
+    //          the display, however it sets the cursor position to row=25. row is 0-based so that would actually put
+    //          it one row beyond the end of the display. This fixes the "Press F1 for"... message at the bottom.
+    // Assume MDA/CGA do not range check because the rows value in the BDA did not exist in the original IBM PC/XT BIOS.
+    if (IS_EGAVGA_ARCH && !IS_PC98_ARCH) {
+        if (nrows && row >= nrows)
+		row = nrows - 1;
+    }
+
     if (IS_DOSV && DOSV_CheckCJKVideoMode()) DOSV_OffCursor();
     else if(J3_IsJapanese()) J3_OffCursor();
     if (page>7) LOG(LOG_INT10,LOG_ERROR)("INT10_SetCursorPos page %d",page);
@@ -1305,8 +1319,6 @@ void INT10_SetCursorPos(uint8_t row,uint8_t col,uint8_t page) {
     // Set the hardware cursor
     uint8_t current=IS_PC98_ARCH ? 0 : real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
     if(page==current) {
-        // Get the dimensions
-        BIOS_NCOLS;
         // Calculate the address knowing nbcols nbrows and page num
         // NOTE: BIOSMEM_CURRENT_START counts in colour/flag pairs
         uint16_t address=(ncols*row)+col+(IS_PC98_ARCH ? 0 : (real_readw(BIOSMEM_SEG,BIOSMEM_CURRENT_START)/2));
@@ -1781,77 +1793,78 @@ void INT10_WriteChar(uint16_t chr,uint8_t attr,uint8_t page,uint16_t count,bool
 }
 
 static void INT10_TeletypeOutputAttr(uint8_t chr,uint8_t attr,bool useattr,uint8_t page) {
-    BIOS_NCOLS;BIOS_NROWS;
-    uint8_t cur_row=CURSOR_POS_ROW(page);
-    uint8_t cur_col=CURSOR_POS_COL(page);
-    switch (chr) {
-    case 8:
-        if(cur_col>0) cur_col--;
-        break;
-    case '\r':
-        cur_col=0;
-        break;
-    case '\n':
-//      cur_col=0; //Seems to break an old chess game
-        cur_row++;
-        break;
-    case 7: /* Beep */
-        // Prepare PIT counter 2 for ~900 Hz square wave
-        IO_Write(0x43, 0xb6);
-        IO_Write(0x42, 0x28);
-        IO_Write(0x42, 0x05);
-        // Speaker on
-        IO_Write(0x61, IO_Read(0x61) | 0x3);
-        // Idle for 1/3rd of a second
-        double start;
-        start = PIC_FullIndex();
-        while ((PIC_FullIndex() - start) < 333.0) CALLBACK_Idle();
-        // Speaker off
-        IO_Write(0x61, IO_Read(0x61) & ~0x3);
-        if (CurMode->type==M_TEXT) {
-            uint16_t chat;
-            INT10_ReadCharAttr(&chat,page);
-            if ((uint8_t)(chat>>8)!=7) return;
-        }
-        chr=' ';
-    default:
-		/* Return if the char code is DBCS at the end of the line (for DOS/V) */
-		if (cur_col + 1 == ncols && IS_DOSV && DOSV_CheckCJKVideoMode() && isKanji1(chr) && prevchr == 0) {
-			INT10_TeletypeOutputAttr(' ', attr, useattr, page);
-			cur_row = CURSOR_POS_ROW(page);
-			cur_col = CURSOR_POS_COL(page);
+	BIOS_NCOLS;BIOS_NROWS;
+	uint8_t cur_row=CURSOR_POS_ROW(page);
+	uint8_t cur_col=CURSOR_POS_COL(page);
+	switch (chr) {
+		case 8:
+			if(cur_col>0) cur_col--;
+			break;
+		case '\r':
+			cur_col=0;
+			break;
+		case '\n':
+//			cur_col=0; // Seems to break an old chess game
+//			           // That's absolutely right, \r CR and \n LF, this is why DOS requires \r\n for newline --J.C>
+			cur_row++;
+			break;
+		case 7: /* Beep */
+			// Prepare PIT counter 2 for ~900 Hz square wave
+			IO_Write(0x43, 0xb6);
+			IO_Write(0x42, 0x28);
+			IO_Write(0x42, 0x05);
+			// Speaker on
+			IO_Write(0x61, IO_Read(0x61) | 0x3);
+			// Idle for 1/3rd of a second
+			double start;
+			start = PIC_FullIndex();
+			while ((PIC_FullIndex() - start) < 333.0) CALLBACK_Idle();
+			// Speaker off
+			IO_Write(0x61, IO_Read(0x61) & ~0x3);
+			if (CurMode->type==M_TEXT) {
+				uint16_t chat;
+				INT10_ReadCharAttr(&chat,page);
+				if ((uint8_t)(chat>>8)!=7) return;
+			}
+			return; /* don't do anything else, not even scrollup on last line (fix for Elder Scrolls Arena installer) */
+		default:
+			/* Return if the char code is DBCS at the end of the line (for DOS/V) */
+			if (cur_col + 1 == ncols && IS_DOSV && DOSV_CheckCJKVideoMode() && isKanji1(chr) && prevchr == 0) {
+				INT10_TeletypeOutputAttr(' ', attr, useattr, page);
+				cur_row = CURSOR_POS_ROW(page);
+				cur_col = CURSOR_POS_COL(page);
+			}
+			/* Draw the actual Character */
+			if(IS_DOSV && DOSV_CheckCJKVideoMode()) {
+				WriteCharV(cur_col,cur_row,chr,attr,useattr);
+			} else {
+				WriteChar(cur_col,cur_row,page,chr,attr,useattr);
+			}
+			cur_col++;
+	}
+	if(cur_col==ncols) {
+		cur_col=0;
+		cur_row++;
+	}
+	// Do we need to scroll ?
+	if(cur_row==nrows) {
+		//Fill with black on non-text modes
+		uint8_t fill = 0;
+		if (IS_PC98_ARCH && CurMode->type == M_TEXT) {
+			//Fill with the default ANSI attribute on textmode
+			fill = DefaultANSIAttr();
 		}
-        /* Draw the actual Character */
-        if(IS_DOSV && DOSV_CheckCJKVideoMode()) {
-            WriteCharV(cur_col,cur_row,chr,attr,useattr);
-        } else {
-            WriteChar(cur_col,cur_row,page,chr,attr,useattr);
-        }
-        cur_col++;
-    }
-    if(cur_col==ncols) {
-        cur_col=0;
-        cur_row++;
-    }
-    // Do we need to scroll ?
-    if(cur_row==nrows) {
-        //Fill with black on non-text modes
-        uint8_t fill = 0;
-        if (IS_PC98_ARCH && CurMode->type == M_TEXT) {
-            //Fill with the default ANSI attribute on textmode
-            fill = DefaultANSIAttr();
-        }
-        else if (CurMode->type==M_TEXT || DOSV_CheckCJKVideoMode()) {
-            //Fill with attribute at cursor on textmode
-            uint16_t chat;
-            INT10_ReadCharAttr(&chat,page);
-            fill=(uint8_t)(chat>>8);
-        }
-        INT10_ScrollWindow(0,0,(uint8_t)(nrows-1),(uint8_t)(ncols-1),-1,fill,page);
-        cur_row--;
-    }
-    // Set the cursor for the page
-    INT10_SetCursorPos(cur_row,cur_col,page);
+		else if (CurMode->type==M_TEXT || DOSV_CheckCJKVideoMode()) {
+			//Fill with attribute at cursor on textmode
+			uint16_t chat;
+			INT10_ReadCharAttr(&chat,page);
+			fill=(uint8_t)(chat>>8);
+		}
+		INT10_ScrollWindow(0,0,(uint8_t)(nrows-1),(uint8_t)(ncols-1),-1,fill,page);
+		cur_row--;
+	}
+	// Set the cursor for the page
+	INT10_SetCursorPos(cur_row,cur_col,page);
 }
 
 void INT10_TeletypeOutputAttr(uint8_t chr,uint8_t attr,bool useattr) {
