@@ -160,6 +160,21 @@ bool isKanji2(uint8_t chr) {
         return (chr >= 0x40 && chr <= 0x7e) || (del_flag && chr == 0x7f) || (chr >= 0x80 && chr <= 0xfc);
 }
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+    bool tfd_isKanji1(uint8_t chr) {
+        return isKanji1(chr);
+    }
+    bool tfd_isKanji2(uint8_t chr) {
+        return isKanji2(chr);
+    }
+#ifdef __cplusplus
+}
+#endif
+
+
+
 static inline int Hex2Int(const char *p) {
     if (*p <= '9')
         return *p - '0';
@@ -418,7 +433,10 @@ bool readPCF(FILE *file, int height) {
   uint16_t lastCol   = read_int16(file);
   uint16_t firstRow  = read_int16(file);
   uint16_t lastRow   = read_int16(file);
-  uint16_t defaultCh = read_int16(file);
+  // uint16_t defaultCh = read_int16(file);
+  fseek(file, 2L, SEEK_CUR);
+  read_bytes += 2L;
+
   if (!(firstCol <= lastCol) || !(firstRow <= lastRow)) {delete[] bitmaps;return false;}
   int nEncodings = (lastCol - firstCol + 1) * (lastRow - firstRow + 1);
   uint16_t *encodings;
@@ -1435,7 +1453,7 @@ void InitFontHandle()
 		if(!font_window) {
 			font_window = XCreateSimpleWindow(font_display, DefaultRootWindow(font_display), 0, 0, 32, 32, 0, BlackPixel(font_display, DefaultScreen(font_display)), WhitePixel(font_display, DefaultScreen(font_display)));
 			font_pixmap = XCreatePixmap(font_display, font_window, 32, 32, DefaultDepth(font_display, 0));
-			font_gc = XCreateGC(font_display, font_pixmap, 0, 0);
+			font_gc = XCreateGC(font_display, font_pixmap, 0, nullptr);
 		}
 	}
 #endif
@@ -2416,7 +2434,7 @@ public:
 		flags = PFLAG_HASROM;
 		bank = 0;
 	}
-	uint8_t readb(PhysPt addr) {
+	uint8_t readb(PhysPt addr) override {
 		uint16_t code;
 		Bitu offset;
 		if(bank == 0) {
@@ -2446,7 +2464,7 @@ public:
 		}
 		return 0;
 	}
-	uint16_t readw(PhysPt addr) {
+	uint16_t readw(PhysPt addr) override {
 		uint16_t code;
 		Bitu offset;
 		if(bank == 0) {
@@ -2476,21 +2494,21 @@ public:
 		}
 		return 0;
 	}
-	uint32_t readd(PhysPt addr) {
+	uint32_t readd(PhysPt addr) override {
         (void)addr;
 		return 0;
 	}
-	void writeb(PhysPt addr,uint8_t val){
+	void writeb(PhysPt addr,uint8_t val) override {
         (void)addr;
 		if((val & 0x80) && val != 0xff) {
 			bank = val & 0x7f;
 		}
 	}
-	void writew(PhysPt addr,uint16_t val){
+	void writew(PhysPt addr,uint16_t val) override {
         (void)addr;
         (void)val;
 	}
-	void writed(PhysPt addr,uint32_t val){
+	void writed(PhysPt addr,uint32_t val) override {
         (void)addr;
         (void)val;
 	}
