@@ -53,10 +53,14 @@ extern int lfn_filefind_handle;
 extern bool ctrlbrk, gbk, rtl, dbcs_sbcs;
 extern bool DOS_BreakFlag, DOS_BreakConioFlag;
 extern uint16_t cmd_line_seg;
+uint8_t prompt_col; // Column position after prompt is displayed
+void WriteChar(uint16_t col, uint16_t row, uint8_t page, uint16_t chr, uint8_t attr, bool useattr);
+
 #if defined(USE_TTF)
 extern bool ttf_dosv;
 #endif
 extern std::map<int, int> pc98boxdrawmap;
+
 void DOS_Shell::ShowPrompt(void) {
 	char dir[DOS_PATHLENGTH];
 	dir[0] = 0; //DOS_GetCurrentDir doesn't always return something. (if drive is messed up)
@@ -103,7 +107,11 @@ void DOS_Shell::ShowPrompt(void) {
 				WriteOut("%d:%02d:%02d.%02d",reg_ch,reg_cl,reg_dh,reg_dl);
 				break;
 			}
+#if !defined(OS2) || !defined(C_SDL2)
 			case 'V': WriteOut("DOSBox-X version %s. Reported DOS version %d.%d.",VERSION,dos.version.major,dos.version.minor); break;
+#else
+			case 'V': WriteOut("DOSBox-X version %s. Reported DOS version %d.%d.",PACKAGE_VERSION,dos.version.major,dos.version.minor); break;
+#endif
 			case '$': WriteOut("$"); break;
 			case '_': WriteOut("\n"); break;
 			case 'M': break;
@@ -112,6 +120,7 @@ void DOS_Shell::ShowPrompt(void) {
 		promptstr++;
 	}
     inshell = false;
+    prompt_col = CURSOR_POS_COL(real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE));
 }
 
 static void outc(uint8_t c) {
@@ -442,6 +451,9 @@ static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_inde
 }
 
 extern bool isDBCSCP();
+void ReadCharAttr(uint16_t col, uint16_t row, uint8_t page, uint16_t* result);
+bool read_lead_byte = false;
+uint8_t temp_lead_byte;
 /* NTS: buffer pointed to by "line" must be at least CMD_MAXLINE+1 large */
 void DOS_Shell::InputCommand(char * line) {
 	Bitu size=CMD_MAXLINE-2; //lastcharacter+0
@@ -596,28 +608,29 @@ void DOS_Shell::InputCommand(char * line) {
             else if (cr == 0x7400) cr = 0x7300;
         }
 #endif
+        uint8_t page, col, row;
         switch (cr) {
             case 0x3d00:	/* F3 */
                 if (!l_history.size()) break;
                 it_history = l_history.begin();
                 if (it_history != l_history.end() && it_history->length() > str_len) {
                     const char *reader = &(it_history->c_str())[str_len];
-                    while ((c = (uint8_t)(*reader++))) {
-                        line[str_index ++] = (char)c;
-                        DOS_WriteFile(STDOUT,&c,&n);
+                    while((c = (uint8_t)(*reader++))) {
+                        line[str_index++] = (char)c;
                     }
                     str_len = str_index = (uint16_t)it_history->length();
                     size = (unsigned int)CMD_MAXLINE - str_index - 2u;
                     line[str_len] = 0;
+                    DOS_WriteFile(STDOUT, (const uint8_t *)it_history->c_str(), &str_len);
                 }
                 break;
 
             case 0x4B00:	/* LEFT */
-                if(IS_PC98_ARCH || (isDBCSCP()
+                if(IS_PC98_ARCH
 #if defined(USE_TTF)
                     && dbcs_sbcs
 #endif
-                    && IS_DOS_JAPANESE)) {
+                    ) {
                     if (str_index) {
                         uint16_t count = GetLastCount(line, str_index);
                         uint8_t ch = line[str_index - 1];
@@ -632,6 +645,15 @@ void DOS_Shell::InputCommand(char * line) {
                             backone();
                         }
                     }
+                    page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                    col = CURSOR_POS_COL(page);
+                    row = CURSOR_POS_ROW(page);
+                    uint16_t get_char, get_char2;
+                    ReadCharAttr(col, row, page, &get_char);
+                    if((line[str_index] & 0xFF) != (uint8_t)(get_char & 0xFF)) {
+                        ReadCharAttr(col-1, row, page, &get_char2);
+                        if((uint8_t)(get_char2 & 0xFF) == (line[str_index] & 0xFF)) INT10_SetCursorPos(row, col - 1, page);
+                    }
                 } else {
                     if (isDBCSCP()
 #if defined(USE_TTF)
@@ -641,8 +663,32 @@ void DOS_Shell::InputCommand(char * line) {
                         backone();
                         str_index --;
                         MoveCaretBackwards();
+                        page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                        col = CURSOR_POS_COL(page);
+                        row = CURSOR_POS_ROW(page);
+                        uint16_t get_char, get_char2;
+                        ReadCharAttr(col, row, page, &get_char);
+                        if((line[str_index] & 0xFF) != (uint8_t)(get_char & 0xFF)) {
+                            ReadCharAttr(col - 1, row, page, &get_char2);
+                            if((uint8_t)(get_char2 & 0xFF) == (line[str_index] & 0xFF)) INT10_SetCursorPos(row, col - 1, page);
+                        }
+                        uint8_t iskanji = 0;
+                        col = CURSOR_POS_COL(page);
+                        if(col > 0 && str_index > 0) {
+                            for(uint16_t i = 0; i < str_index; i++) {
+                                if(isKanji1(line[i]) && isKanji2(line[i + 1])) {
+                                    if(i + 1 == str_index) iskanji = 2;
+                                    else i++; // skip next character
+                                }
+                            }
+
+                            if(iskanji == 2) {
+                                backone();
+                                str_index--;
+                            }
+                        }
                     }
-                    if (str_index) {
+                    else if (str_index) {
                         backone();
                         str_index --;
                         MoveCaretBackwards();
@@ -711,15 +757,31 @@ void DOS_Shell::InputCommand(char * line) {
                             count--;
                         }
                     }
+                    page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                    col = CURSOR_POS_COL(page);
+                    BIOS_NCOLS;
+                    if(col == ncols - 1 && str_index < str_len && isKanji1(line[str_index]) && isKanji2(line[str_index+1])) {
+                        row = CURSOR_POS_ROW(page);
+                        INT10_SetCursorPos(row+1, 0, page);
+                    }
                 } else {
                     if (isDBCSCP()
 #if defined(USE_TTF)
                         &&dbcs_sbcs
 #endif
                         &&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index+1]>=0x40))) {
-                        outc((uint8_t)line[str_index++]);
+                        if(isKanji1((uint8_t)line[str_index]) && isKanji2((uint8_t)line[str_index + 1])) {
+                            const uint8_t buf[2] = {(uint8_t)line[str_index],(uint8_t)line[str_index + 1]};
+                            uint16_t num = 2;
+                            DOS_WriteFile(STDOUT, buf, &num);
+                            str_index += 2;
+                        }
+                        else {
+                            outc((uint8_t)line[str_index]);
+                            str_index++;
+                        }
                     }
-                    if (str_index < str_len) {
+                    else if (str_index < str_len) {
                         outc((uint8_t)line[str_index++]);
                     }
                 }
@@ -730,6 +792,12 @@ void DOS_Shell::InputCommand(char * line) {
                     backone();
                     str_index--;
                 }
+                page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                col = CURSOR_POS_COL(page);
+                while(col > prompt_col) {
+                    backone();
+                    col = CURSOR_POS_COL(page);
+                }
                 break;
 
             case 0x5200:    /* INS */
@@ -742,8 +810,11 @@ void DOS_Shell::InputCommand(char * line) {
                 break;
 
             case 0x4F00:	/* END */
-                while (str_index < str_len) {
-                    outc((uint8_t)line[str_index++]);
+                {
+                    uint16_t a = str_len - str_index;
+                    uint8_t* text = reinterpret_cast<uint8_t*>(&line[str_index]);
+                    DOS_WriteFile(STDOUT, text, &a);//write buffer to screen
+                    str_index = str_len;
                 }
                 break;
 
@@ -765,6 +836,13 @@ void DOS_Shell::InputCommand(char * line) {
                     // removes all characters
                     backone(); outc(' '); backone();
                 }
+                page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                col = CURSOR_POS_COL(page);
+                while(col > prompt_col){
+                    backone(); outc(' '); backone();
+                    col = CURSOR_POS_COL(page);
+                }
+
                 strcpy(line, it_history->c_str());
                 len = (uint16_t)it_history->length();
                 str_len = str_index = len;
@@ -799,6 +877,13 @@ void DOS_Shell::InputCommand(char * line) {
                     // removes all characters
                     backone(); outc(' '); backone();
                 }
+                page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                col = CURSOR_POS_COL(page);
+                while(col > prompt_col) {
+                    backone(); outc(' '); backone();
+                    col = CURSOR_POS_COL(page);
+                }
+
                 strcpy(line, it_history->c_str());
                 len = (uint16_t)it_history->length();
                 str_len = str_index = len;
@@ -808,35 +893,56 @@ void DOS_Shell::InputCommand(char * line) {
 
                 break;
             case 0x5300:/* DELETE */
-                if(IS_PC98_ARCH || (isDBCSCP()
+                if(IS_PC98_ARCH
 #if defined(USE_TTF)
                     && dbcs_sbcs
 #endif
-                    && IS_DOS_JAPANESE)) {
+                    ) {
                     if(str_len) {
                         size += DeleteBackspace(true, line, str_index, str_len);
                     }
                 } else {
                     if(str_index>=str_len) break;
-                    int k=1;
+                    uint8_t k=1;
                     if (isDBCSCP()
 #if defined(USE_TTF)
                         &&dbcs_sbcs
 #endif
                         &&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index+1]>=0x40)))
                         k=2;
-                    for (int i=0; i<k; i++) {
-                        uint16_t a=str_len-str_index-1;
-                        uint8_t* text=reinterpret_cast<uint8_t*>(&line[str_index+1]);
-                        DOS_WriteFile(STDOUT,text,&a);//write buffer to screen
-                        outc(' ');backone();
-                        for(Bitu i=str_index;i<(str_len-1u);i++) {
-                            line[i]=line[i+1u];
-                            backone();
-                        }
-                        line[--str_len]=0;
-                        size++;
+                    for(uint16_t i = str_index; i < (str_len - k); i++) {
+                        line[i] = line[i + k];
                     }
+                    line[str_len - k] = 0;
+                    line[str_len - k + 1] = 0;
+                    line[str_len] = 0;
+                    str_len -= k;
+                    uint16_t a=str_len-str_index;
+                    uint8_t* text=reinterpret_cast<uint8_t*>(&line[str_index]);
+                    page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                    BIOS_NCOLS; BIOS_NROWS;
+                    col = CURSOR_POS_COL(page);
+                    DOS_WriteFile(STDOUT,text,&a);//write buffer to screen
+                    uint16_t col2 = CURSOR_POS_COL(page);
+                    row = CURSOR_POS_ROW(page);
+                    if(k == 2){
+                        outc(' '); outc(' '); backone(); backone();
+                    }
+                    else {
+                        outc(' '); backone();
+                    }
+                    if(col2 >= ncols - (k==2?3:1) && row < nrows) {
+                        for(int i = 0; i <= k; i++) WriteChar(i, row + 1, page, ' ', 0, false);
+                    }
+                    col2 = CURSOR_POS_COL(page); //LOG_MSG("col=%d, a=%d", col,a);
+                    if(col2 >= a) INT10_SetCursorPos(row, col2 - a, page);
+                    else {
+                        uint16_t lines_up = (a - col2 - 1) / ncols + 1;
+                        if(col2 < ncols) for(int i = col2; i < ncols; i++) WriteChar(i, row, page, ' ', 0, false);
+                        if(row >= lines_up) INT10_SetCursorPos(row - lines_up, col, page);
+                        else INT10_SetCursorPos(0, 0, page);
+                    }
+                    size += k;
                 }
                 break;
             case 0x0F00:	/* Shift-Tab */
@@ -865,42 +971,70 @@ void DOS_Shell::InputCommand(char * line) {
                 }
                 break;
             case 0x08:				/* BackSpace */
-                if(IS_PC98_ARCH || (isDBCSCP()
+                if(IS_PC98_ARCH
 #if defined(USE_TTF)
                     && dbcs_sbcs
 #endif
-                    && IS_DOS_JAPANESE)) {
+                    ) {
                     if(str_index) {
                         size += DeleteBackspace(false, line, str_index, str_len);
                     }
-                } else {
-                    int k=1;
-                    if (isDBCSCP()
+                }
+                else {
+                    if(str_index == 0)break;
+                    page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                    uint8_t bytes = 1; // Previous character is DBCS or SBCS
+                    if(isDBCSCP()
 #if defined(USE_TTF)
-                        &&dbcs_sbcs
+                        && dbcs_sbcs
 #endif
-                        &&str_index>1&&(line[str_index-1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index-1]>=0x40))&&line[str_index-2]<0)
-                        k=2;
-                    for (int i=0; i<k; i++)
-                        if (str_index) {
+                        ) bytes = GetLastCount(line, str_index);
+                    
+                    if(str_index < str_len) {
+                        uint16_t a = str_len - str_index;
+                        uint8_t* text = reinterpret_cast<uint8_t*>(&line[str_index]);
+                        DOS_WriteFile(STDOUT, text, &a);
+                    }
+                    backone();
+                    if(bytes == 2 && str_len >= 2) backone();
+                    outc(' ');
+                    if(bytes == 2 && str_len >= 2) outc(' ');
+                    backone();
+                    if(bytes == 2 && str_len >= 2) backone();
+
+                    if(str_index < str_len)
+                        memmove(&line[str_index - bytes], &line[str_index], str_len - str_index);  // Shift buffer to left
+
+                    line[str_len - 1] = '\0';
+                    if(bytes == 2)line[str_len - 2] = '\0';
+                    str_len -= bytes;
+                    str_index -= bytes;
+                    size -= bytes;
+
+                    for(int i = str_len; i > 0; i--) backone();
+                    col = CURSOR_POS_COL(page);
+                    while(col > prompt_col) {
+                        backone();
+                        col = CURSOR_POS_COL(page);
+                    }
+
+                    uint16_t a = str_len;
+                    uint8_t* text = reinterpret_cast<uint8_t*>(&line[0]);
+                    DOS_WriteFile(STDOUT, text, &a); // Rewrite the command history
+                    outc(' '); outc(' ');  backone(); backone();
+
+                    if(str_index < str_len) {
+                        for(int i = str_len; i > 0; i--) backone();
+                        col = CURSOR_POS_COL(page);
+                        while(col > prompt_col) {
                             backone();
-                            uint32_t str_remain=(uint32_t)(str_len - str_index);
-                            size++;
-                            if (str_remain) {
-                                memmove(&line[str_index-1],&line[str_index],str_remain);
-                                line[--str_len]=0;
-                                str_index --;
-                                /* Go back to redraw */
-                                for (uint16_t i=str_index; i < str_len; i++)
-                                    outc((uint8_t)line[i]);
-                            } else {
-                                line[--str_index] = '\0';
-                                str_len--;
-                            }
-                            outc(' ');	backone();
-                            // moves the cursor left
-                            while (str_remain--) backone();
+                            col = CURSOR_POS_COL(page);
                         }
+
+                        a = str_index;
+                        text = reinterpret_cast<uint8_t*>(&line[0]);
+                        DOS_WriteFile(STDOUT, text, &a); // Move to the cursor position
+                    }
                 }
                 if (l_completion.size()) l_completion.clear();
                 break;
@@ -916,6 +1050,12 @@ void DOS_Shell::InputCommand(char * line) {
                 DOS_BreakConioFlag = false;
                 break;
             case 0x0d:				/* Don't care, and return */
+            {
+                uint16_t a = str_len - str_index;
+                uint8_t* text = reinterpret_cast<uint8_t*>(&line[str_index]);
+                DOS_WriteFile(STDOUT, text, &a);//goto end of command line
+                str_index = str_len;
+            }
                 if(!echo) { outc('\r'); outc('\n'); }
                 size=0;			//Kill the while loop
                 break;
@@ -1059,11 +1199,12 @@ void DOS_Shell::InputCommand(char * line) {
                 str_len = 0;
                 break;
             default:
-                if(IS_PC98_ARCH || (isDBCSCP()
+                if(IS_PC98_ARCH  
 #if defined(USE_TTF)
                     && dbcs_sbcs
 #endif
-                    && IS_DOS_JAPANESE)) {
+                    ) {
+                    if(dos.loaded_codepage == 932 && isKanji1(cr >> 8) && (cr & 0xFF) == 0) break;
                     bool kanji_flag = false;
                     uint16_t pos = str_index;
                     while(1) {
@@ -1101,30 +1242,96 @@ void DOS_Shell::InputCommand(char * line) {
                         }
                     }
                 } else {
-                    if (cr >= 0x100) break;
-                    if (l_completion.size()) l_completion.clear();
-                    if(str_index < str_len && !INT10_GetInsertState()) { //mem_readb(BIOS_KEYBOARD_FLAGS1)&0x80) dev_con.h ?
-                        outc(' ');//move cursor one to the right.
-                        uint16_t a = str_len - str_index;
-                        uint8_t* text=reinterpret_cast<uint8_t*>(&line[str_index]);
-                        DOS_WriteFile(STDOUT,text,&a);//write buffer to screen
-                        backone();//undo the cursor the right.
-                        for(Bitu i=str_len;i>str_index;i--) {
-                            line[i]=line[i-1]; //move internal buffer
-                            backone(); //move cursor back (from write buffer to screen)
+                    if(cr >= 0x100) break;
+                    if(l_completion.size()) l_completion.clear();
+
+                    page = real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_PAGE);
+                    if(str_index < str_len) {  // Insert Mode Handling
+                        col = CURSOR_POS_COL(page);
+                        bool delete_lead_byte = false;
+                        if(INT10_GetInsertState()) {
+                            for(uint16_t i = str_len; i > str_index; i--) {
+                                line[i] = line[i - 1];  // Shift buffer to the right
+                            }
+                            line[++str_len] = '\0';  // New buffer end
+                            size--;
                         }
-                        line[++str_len]=0;//new end (as the internal buffer moved one place to the right
-                        size--;
+                        else {
+                            if(isDBCSCP()) {
+                                if(read_lead_byte) {
+                                    if(!isKanji1(line[str_index])) {
+                                        for(uint16_t i = str_len; i > str_index; i--) {
+                                            line[i] = line[i - 1];  // Shift buffer to the right
+                                        }
+                                        line[++str_len] = '\0';  // New buffer end
+                                        size--;
+                                    }
+                                    line[str_index] = temp_lead_byte;
+                                    line[str_index + 1] = (char)(cr & 0xFF);
+                                }
+                                else if(!isKanji1(cr & 0xFF) && isKanji1(line[str_index])) {
+                                    for(uint16_t i = str_index + 1; i < str_len; i++) {
+                                        line[i] = line[i + 1];  // Delete trailing byte of DBCS since leading byte is overwritten
+                                    }
+                                    read_lead_byte = false;
+                                    delete_lead_byte = true;
+                                    line[str_len] = ' ';
+                                    size++;
+                                }
+                                else if(isKanji1(cr & 0xFF)) {
+                                    temp_lead_byte = (char)(cr & 0xFF);
+                                    read_lead_byte = true;
+                                    continue;
+                                }
+                            }
+                            else read_lead_byte = false;
+                        }
+                        if(!read_lead_byte)line[str_index] = (char)(cr & 0xFF);
+                        uint16_t a = str_len - str_index;
+                        uint8_t* text = reinterpret_cast<uint8_t*>(&line[str_index]);
+                        DOS_WriteFile(STDOUT, text, &a);  // Write remaining buffer to screen
+                        str_index++;
+                        if(read_lead_byte) {
+                            str_index++;
+                            col++;
+                            read_lead_byte = false;
+                        }
+                        if(delete_lead_byte) {
+                            line[str_len] = '\0';  // Ensure null-terminated
+                            str_len--;
+                            outc(' '); outc(' '); backone(); backone();
+                        }
+
+                        for(uint16_t i = str_len; i > str_index; i--) {
+                            backone();
+                        }
+                        uint8_t col2 = CURSOR_POS_COL(page);
+                        while(col2 > col+1) {
+                            backone();
+                            col2 = CURSOR_POS_COL(page);
+                        }
+                        break;
                     }
 
-                    line[str_index]=(char)(cr&0xFF);
-                    str_index ++;
-                    if (str_index > str_len){ 
-                        line[str_index] = '\0';
+                    // Insert the new character
+                    line[str_index] = (char)(cr & 0xFF);
+                    str_index++;
+                    if(str_index > str_len) {
+                        line[str_index] = '\0';  // Null-terminate if new end
                         str_len++;
                         size--;
                     }
-                    DOS_WriteFile(STDOUT,&c,&n);
+                    else if(!INT10_GetInsertState()) {
+                        // Overwrite mode: just move to the next position
+                        if(str_index == str_len) {
+                            line[str_index] = '\0';  // Ensure null-terminated
+                            str_len++;
+                            size--;
+                        }
+                    }
+
+                    // Output the inserted character
+                    outc((uint8_t)(cr & 0xFF));
                 }
                 break;
         }
@@ -1264,7 +1471,7 @@ bool DOS_Shell::Execute(char* name, const char* args) {
 
 	const Section_prop* sec = static_cast<Section_prop*>(control->GetSection("dos"));
 	/* check for a drive change */
-	if (((strcmp(name + 1, ":") == 0) || (strcmp(name + 1, ":\\") == 0)) && isalpha(*name) && !control->SecureMode())
+	if (((strcmp(name + 1, ":") == 0) || (strcmp(name + 1, ":\\") == 0)) && isalpha(*name))
 	{
 #ifdef WIN32
 		uint8_t c;uint16_t n;
